package com.gosingapore.common.enums.ai;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;

import java.util.regex.Pattern;

/**
 * <p>
 * 范围类型
 * </p>
 *
 * @author ZhangJinyu
 * @since 2020-01-15
 */
public enum RangeTypeEnum {
    /**
     * 无类型
     */
    NONE(null, null, null),
    /**
     * 整数范围
     */
    INTEGER(Pattern.compile("^[0-9]\\d*-$"), Pattern.compile("^[0-9]\\d*-[0-9]\\d*"), (Pattern.compile("^[0-9]\\d*\\+$"))),
    /**
     * 时间范围
     */
    TIME(null, Pattern.compile("(([0-1][0-9])|([2][0-4])|([0-2][0-3])):([0-5][0-9])-([0-5][0-9]):(([0-1][0-9])|([2][0-4])|([0-5][0-9]))"), null);
    Pattern minRegex;
    Pattern middleRegex;
    Pattern maxRegex;
    public static final String SEPARATOR = "-";
    public static final String MIN = "-";
    public static final String MAX = "+";

    RangeTypeEnum(Pattern minRegex, Pattern middleRegex, Pattern maxRegex) {
        this.minRegex = minRegex;
        this.middleRegex = middleRegex;
        this.maxRegex = maxRegex;
    }

    public static RangeTypeEnum get(String rangeType) {
        if (!StringUtils.isEmpty(rangeType)) {
            for (RangeTypeEnum rangeTypeEnum : values()) {
                if (rangeTypeEnum.name().equalsIgnoreCase(rangeType)) {
                    return rangeTypeEnum;
                }
            }
        }
        return NONE;
    }

    public boolean isNone() {
        return this.equals(NONE);
    }

    public boolean verifyValue(String value) {
        if (StringUtils.isEmpty(value)) {
            return true;
        }
        if (this.equals(NONE)) {
            return true;
        }
        if (minRegex != null) {
            if (minRegex.matcher(value).matches()) {
                return true;
            }
        }
        if (middleRegex != null) {
            if (middleRegex.matcher(value).matches()) {
                return true;
            }
        }
        if (maxRegex != null) {
            return maxRegex.matcher(value).matches();
        }
        return false;
    }

    public boolean match(String realValueForJob, String realValueForCustomer) {
        if (this.equals(TIME)) {
            if (!verifyValue(realValueForJob) || !verifyValue(realValueForCustomer)) {
                return false;
            }
            return timeRangeIn(realValueForCustomer, realValueForJob);
        } else if (this.equals(INTEGER)) {
            if (!NumberUtils.isParsable(realValueForCustomer)) {
                if (!verifyValue(realValueForCustomer)) {
                    return false;
                }
            }
            if (!NumberUtils.isParsable(realValueForJob)) {
                if (!verifyValue(realValueForJob)) {
                    return false;
                }
            }
            return intRangeIn(realValueForCustomer, realValueForJob);
        } else {
            return false;
        }
    }

    public boolean isMin(String value) {
        return minRegex != null && minRegex.matcher(value).matches();
    }

    public boolean isMiddle(String value) {
        return middleRegex != null && middleRegex.matcher(value).matches();
    }

    public boolean isMax(String value) {
        return maxRegex != null && maxRegex.matcher(value).matches();
    }

    private boolean intRangeIn(String realValueForCustomer, String realValueForJob) {
        if (isMin(realValueForCustomer)) {
            int customerValue = Integer.parseInt(realValueForCustomer.replace(MIN, ""));
            if (isMin(realValueForJob)) {
                int jobValue = Integer.parseInt(realValueForJob.replace(MIN, ""));
                return customerValue < jobValue;
            }
            if (isMiddle(realValueForJob)) {
                String[] split = realValueForJob.split(SEPARATOR);
                int startValue = Integer.parseInt(split[0]);
                if (customerValue >= startValue) {
                    return true;
                }
            }
            if (isMax(realValueForJob)) {
                int value = Integer.parseInt(realValueForJob.replace(MAX, ""));
                return customerValue <= value;
            }
        } else if (isMiddle(realValueForCustomer)) {
            String[] splitCustomerValue = realValueForCustomer.split(SEPARATOR);
            int customerStartValue = Integer.parseInt(splitCustomerValue[0]);
            int customerEndValue = Integer.parseInt(splitCustomerValue[1]);
            if (isMin(realValueForJob)) {
                int jobValue = Integer.parseInt(realValueForJob.replace(MIN, ""));
                return customerStartValue <= jobValue;
            }
            if (isMiddle(realValueForJob)) {
                String[] split = realValueForJob.split(SEPARATOR);
                int startValue = Integer.parseInt(split[0]);
                int endValue = Integer.parseInt(split[1]);
                if (customerStartValue <= startValue || customerStartValue <= endValue) {
                    return true;
                }
            }
            if (isMax(realValueForJob)) {
                int jobValue = Integer.parseInt(realValueForJob.replace(MAX, ""));
                return customerStartValue <= jobValue || customerEndValue >= jobValue;
            }
        } else if (isMax(realValueForCustomer)) {
            int customerValue = Integer.parseInt(realValueForCustomer.replace(MAX, ""));
            if (isMin(realValueForJob)) {
                int jobValue = Integer.parseInt(realValueForJob.replace(MIN, ""));
                return customerValue < jobValue;
            }
            if (isMiddle(realValueForJob)) {
                String[] split = realValueForJob.split(SEPARATOR);
                int endValue = Integer.parseInt(split[1]);
                if (customerValue <= endValue) {
                    return true;
                }
            }
            if (isMax(realValueForJob)) {
                int value = Integer.parseInt(realValueForJob.replace(MAX, ""));
                return customerValue <= value;
            }
        } else {
            if (NumberUtils.isParsable(realValueForCustomer)) {
                int i = NumberUtils.toInt(realValueForCustomer);
                if (isMin(realValueForJob)) {
                    int jobValue = Integer.parseInt(realValueForJob.replace(MIN, ""));
                    return i <= jobValue;
                }
                if (isMiddle(realValueForJob)) {
                    String[] split = realValueForJob.split(SEPARATOR);
                    int startValue = Integer.parseInt(split[0]);
                    int endValue = Integer.parseInt(split[1]);
                    if (i >= startValue && i <= endValue) {
                        return true;
                    }
                }
                if (isMax(realValueForJob)) {
                    int value = Integer.parseInt(realValueForJob.replace(MAX, ""));
                    return i >= value;
                }
            }
        }
        return false;
    }

    private boolean timeRangeIn(String timeRangeSource, String timeRangeTarget) {
        if (StringUtils.isBlank(timeRangeSource) || StringUtils.isBlank(timeRangeTarget)) {
            return false;
        }
        String[] source = timeRangeSource.replace(":", "").split("-");
        int sourceTime0 = Integer.parseInt(source[0]);
        int sourceTime1 = Integer.parseInt(source[1]);
        String[] target = timeRangeTarget.replace(":", "").split("-");
        int targetTime0 = Integer.parseInt(target[0]);
        int targetTime1 = Integer.parseInt(target[1]);
        /*例如:18:00-02:00*/
        if (targetTime0 > targetTime1) {
            /*例如:17:00-01:00 或者 19:00-03:00*/
            if (sourceTime0 >= sourceTime1) {
                return sourceTime0 >= targetTime0;
            } else {
                return sourceTime0 >= targetTime0 || sourceTime1 <= targetTime1;
            }
        } else {
            if (sourceTime0 >= sourceTime1) {
                return sourceTime0 >= targetTime1;
            } else {
                return sourceTime0 >= targetTime1 || sourceTime1 <= targetTime0;
            }
        }
    }
}
