package com.gosingapore.common.event.robot.job.listener;

import com.cloud.gs.hiring.base.util.ResponseCode;
import com.gosingapore.common.enums.dict.Dict107JobTypeEnum;
import com.gosingapore.common.enums.dict.Dict38CountryEnum;
import com.cloud.gs.hiring.common.event.robot.job.JobOnlineEvent;
import com.cloud.gs.hiring.common.event.robot.job.RobotSendingLimit;
import com.gosingapore.common.exception.CustomizeException;
import com.cloud.gs.hiring.common.mapper.GsCountryPassJobMapper;
import com.cloud.gs.hiring.common.mapper.GsJobMapper;
import com.cloud.gs.hiring.common.mapper.job.JobOnlineRobotMsgMapper;
import com.gosingapore.common.model.dto.job.JobCountryPassDto;
import com.cloud.gs.hiring.common.model.entity.GsJob;
import com.cloud.gs.hiring.common.model.wechat.WeChatRobotResponse;
import com.gosingapore.common.properties.WeChatGroupTemplateWebhook;
import com.gosingapore.common.properties.robot.WeChatRobotProperties;
import com.gosingapore.common.util.UrlParamGetter;
import com.gosingapore.common.util.WeChatRobotSender;
import com.gosingapore.common.util.job.RobotOfJobOnlineWords;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.transaction.event.TransactionPhase;
import org.springframework.transaction.event.TransactionalEventListener;
import org.springframework.util.CollectionUtils;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author: QiWenXiao
 */
@Component
@Slf4j
public class JobOnlineListener {

    /**
     * 解释下这里为什么用字符串而不用 {@link Dict38CountryEnum}
     * 因为我觉得这个枚举有可能会变，就这么几个国家，却用了好几个枚举来表示（工人国籍、电文国籍等）
     * 但是一个国家的英文单词是一定不会变的
     * 如果枚举变了，在 xml 文件写死的值就会判断失误
     */
    private static final String NATIONALITY_MALAYSIA = "Malaysia";
    private static final String NATIONALITY_CHINA = "China";


    /**
     * 调用企业微信机器人接口
     **/
    private static final RestTemplate REST_TEMPLATE = new RestTemplate();
    @Resource
    private GsCountryPassJobMapper countryPassJobMapper;

    @Resource
    private JobOnlineRobotMsgMapper jobOnlineRobotMsgMapper;

    @Resource
    private WeChatRobotProperties robotProperties;

    @Resource
    private RedisTemplate<String,Integer> redisTemplate;
    /**
     * 企业微信机器人单个机器人发送次数记录和职位上线次数记录
     */
    @Resource
    private RobotSendingLimit robotSendingLimit;
    @Resource
    private GsJobMapper gsJobMapper;
    @TransactionalEventListener(value = JobOnlineEvent.class,phase = TransactionPhase.BEFORE_COMMIT)
    @Async
    public void listenJobOnline(JobOnlineEvent event) {
        Integer jobId = (Integer) event.getSource();
        // 查询出国籍和准证，并用来判断发送消息到哪个群里
        JobCountryPassDto jobCountryPassDto = countryPassJobMapper.selectJobPassAndCountry(jobId);
        // 校验国籍和准证类型的值是否为空
        if (CollectionUtils.isEmpty(jobCountryPassDto.getCountryCode())
                || CollectionUtils.isEmpty(jobCountryPassDto.getPassCode())) {
            log.info("职位ID为{}的工人国籍或准证类型的值为空", jobId);
            throw new CustomizeException(ResponseCode.SYS_INTERNAL_SERVER_ERROR, "职位ID=" + jobId + "的工人国籍或准证类型的值为空", "");
        }

        List<String> country = jobCountryPassDto.getCountryValue().stream().collect(Collectors.toList());
        String countryValue = String.join("/", country);

        Integer typeValue = jobCountryPassDto.getTypeValue().get(0);

        String typeName = jobCountryPassDto.getTypeName().get(0);

        Dict107JobTypeEnum jobTypeEnum = Dict107JobTypeEnum.get(jobCountryPassDto.getJobType());
        // 要发送到哪个国家的群，目前只发送到中国和马来
        List<Dict38CountryEnum> nationalityToBeSend = new ArrayList<>(2);
        // 职位国籍和准证类型对照表：https://docs.qq.com/sheet/DS3pZakdTVE5qU3Zz?tab=BB08J2，下面这两个 if 就是根据这个表中的信息判断的
        // 发到马来群
        if (jobOnlineRobotMsgMapper.selectCountryByJobId(jobId, Dict38CountryEnum.MALAYSIA.getValue()) > 0) {
            nationalityToBeSend.add(Dict38CountryEnum.MALAYSIA);
        }
        // 发到中国群
        if (jobOnlineRobotMsgMapper.selectCountryByJobId(jobId, Dict38CountryEnum.CHINA.getValue()) > 0) {
            nationalityToBeSend.add(Dict38CountryEnum.CHINA);
        }
        nationalityToBeSend.forEach( nationality -> robotSendMsg(jobId, jobTypeEnum, nationality,countryValue,typeValue,typeName));
        // 发完消息将职位更新成旧单
        GsJob gsJob = gsJobMapper.selectById(jobId).setNewOrder(Boolean.FALSE);
        gsJobMapper.updateById(gsJob);
    }

    /**
     * 职位上线发送消息
     *
     * @param jobId   职位ID
     * @param jobType 职位类型
     * @param country 判断发送到中国群还是马来群
     */
    public void robotSendMsg(Integer jobId, Dict107JobTypeEnum jobType, Dict38CountryEnum country,String countryValue,Integer typeValue,String typeName) {
        // 单条职位机器人发送上限
        Integer theNumberOfASingleMessageToBeSentToTheGroup = robotProperties.getTheNumberOfASingleMessageToBeSentToTheGroup();
        // 能否继续发送消息
        boolean isCanSendOrNot;
        if (Dict38CountryEnum.CHINA.equals(country)) {
            isCanSendOrNot = robotSendingLimit.ch().getSentCountByJobId(jobId) >= robotProperties.getLimitChTheNumberOfJobOnline() * theNumberOfASingleMessageToBeSentToTheGroup;
        } else {
            // 马来只有一个群，而且只能在首次上线时发机器人消息
            isCanSendOrNot = robotSendingLimit.ml().getSentCountByJobId(jobId) >= robotProperties.getLimitMlTheNumberOfJobOnline();
        }
        // 校验这个职位今天上线操作几次了，消息限定次数 乘以 群的个数
        if (isCanSendOrNot) {
            log.info("职位ID={}的职位今天已经超过发布次数了，今日不再到企业微信中发消息", jobId);
            return;
        }
        RobotOfJobOnlineWords robotWords = getRobotWordsByJobType(jobId, jobType, country);
        robotWords.setCountry(countryValue + "单");
        if(typeValue == 5){
            robotWords.setTypeName(typeName);
        }
        //统计新单得数量
        if((country.getValue().equals(Dict38CountryEnum.CHINA.getValue())
                || country.getValue().equals(Dict38CountryEnum.VIETNAM.getValue()))
                && robotWords.getNewOrder()){
            Integer sendCount = redisTemplate.opsForValue().get("report:ch:count");
            if(sendCount == null){
                sendCount = 0;
            }
            sendCount += 1;
            Long expiresTime = getSecondsNextEarlyMorning();
            redisTemplate.opsForValue().set("report:ch:count",sendCount,expiresTime, TimeUnit.SECONDS);
            robotWords.setCount(sendCount);
        }
        List<WeChatGroupTemplateWebhook> templateWebhooks = robotProperties.getTemplateWebhooks();
        for (WeChatGroupTemplateWebhook templateWebhook : templateWebhooks) {
            if (country.getValue().equals(templateWebhook.getNationalityCode())) {
                doSend(templateWebhook.getWebhooks(),0,robotWords.toMarkdown(templateWebhook.getTemplate()), jobId, country);
            }
        }

    }

    /**
     * 判断当前时间距离第二天凌晨的秒数
     *
     * @return 返回值单位为[s:秒]
     */
    public  Long getSecondsNextEarlyMorning() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DAY_OF_YEAR, 1);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return (cal.getTimeInMillis() - System.currentTimeMillis()) / 1000;
    }

    /**
     * 机器人发送消息
     *
     * @param robotWebhooks     机器人调用地址
     * @param robotWebhookIndex 当前用哪个机器人
     * @param jobId             职位ID
     * @return 是否发送成功
     */
    private boolean doSend(List<String> robotWebhooks, Integer robotWebhookIndex, String content, Integer jobId, Dict38CountryEnum country) {
        // 防止下标越界，进入这个 if 代表没有机器人可用了
        if (CollectionUtils.isEmpty(robotWebhooks) || robotWebhooks.size() - 1 < robotWebhookIndex) {
            return false;
        }
        // 企业微信返回的状态码
        ResponseEntity<WeChatRobotResponse> robotResponse = WeChatRobotSender.send(robotWebhooks.get(robotWebhookIndex),content);
        String errCode = robotResponse.getBody().getErrcode();
        // 返回码为0代表发送成功
        if ("0".equals(errCode)) {
            if (Dict38CountryEnum.CHINA.equals(country)) {
                robotSendingLimit.ch();
            } else {
                robotSendingLimit.ml();
            }
            robotSendingLimit.recordJobOnlineCount(jobId, country);
            robotSendingLimit.recordRobotSendCount(UrlParamGetter.getUrl(robotWebhooks.get(robotWebhookIndex), "key"));
            return true;
            // 如果机器人返回的错误码是 45009 就换个机器人
        } else if ("45009".equals(errCode)) {
            // 找到能发送消息的机器人
            for (int i = 0; i < robotWebhooks.size(); i++) {
                String robotWebhook = robotWebhooks.get(i);
                int robotSendCount = robotSendingLimit.getSendCountByRobotKey(UrlParamGetter.getUrl(robotWebhook, "key"));
                if (robotSendCount < 20) {
                    robotWebhookIndex = i;
                    break;
                }
            }
            doSend(robotWebhooks, robotWebhookIndex, content, jobId, country);
        }
        return false;
    }


    /**
     * 根据职位ID 和职位类型获取机器人话术
     *
     * @param jobId   职位ID
     * @param jobType 职位类型
     * @return 机器人话术
     */
    private RobotOfJobOnlineWords getRobotWordsByJobType(Integer jobId, Dict107JobTypeEnum jobType, Dict38CountryEnum country) {
        RobotOfJobOnlineWords robotWords;
        String nationality = null;
        if (Dict38CountryEnum.MALAYSIA.equals(country)) {
            nationality = NATIONALITY_MALAYSIA;
        } else if (Dict38CountryEnum.CHINA.equals(country)) {
            nationality = NATIONALITY_CHINA;
        }
        switch (jobType) {
            case SELF_STANDARD:
                robotWords = jobOnlineRobotMsgMapper.selectRobotWordsOfNewOrder(jobId, nationality);
                robotWords.setUrl(robotProperties.getSelfStandardUrl());
                break;
            case SELF_NONSTANDARD:
                robotWords = jobOnlineRobotMsgMapper.selectRobotWordsOfSecondHandOrder(jobId, nationality);
                robotWords.setUrl(robotProperties.getSelfNonstandardUrl());
                break;
            case PARTNER_STANDARD:
                robotWords = jobOnlineRobotMsgMapper.selectRobotWordsOfPartnerStandard(jobId);
                robotWords.setUrl(robotProperties.getPartnerStandardUrl());
                break;
            case PARTNER_NONSTANDARD:
                robotWords = jobOnlineRobotMsgMapper.selectRobotWordsOfPartnerNonstandard(jobId);
                robotWords.setUrl(robotProperties.getPartnerNonstandardUrl());
                break;
            default:
                throw new CustomizeException(ResponseCode.SYS_INTERNAL_SERVER_ERROR, "未知职位类型", "unknown job type");
        }
        // 找到职位中存储的实际工作场景照片和工作视频或者宣传视频
        robotWords.setMediaUrlList(jobOnlineRobotMsgMapper.selectMediaUrlByJobId(jobId));
        return robotWords;
    }


}
