package com.gosingapore.common.processor;

import com.cloud.gs.hiring.base.util.ApiResult;
import com.cloud.gs.hiring.base.util.ResponseCode;
import com.gosingapore.common.exception.CustomizeException;
import com.gosingapore.common.util.EsCustomerContext;
import com.gosingapore.common.util.EsJobContext;
import com.gosingapore.common.util.MessageRemindContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.method.annotation.MethodArgumentTypeMismatchException;
import org.springframework.web.multipart.MaxUploadSizeExceededException;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.nio.file.AccessDeniedException;
import java.util.Set;

import static com.cloud.gs.hiring.base.util.ResponseCode.*;

/**
 * 全局异常处理器
 *
 * @author shuyan
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {

    @Resource
    private MessageRemindContext messageRemindContext;
    @Resource
    private EsJobContext esJobContext;
    @Resource
    private EsCustomerContext esCustomerContext;

    @ExceptionHandler(MethodArgumentTypeMismatchException.class)
    public ResponseEntity<ApiResult<?>> methodArgumentTypeMismatchException(MethodArgumentTypeMismatchException e) {
        clearCacheRTCMessage();
        log.warn("无效的请求参数: ", e);
        ResponseCode instance = instance(COMMON_INVALID_PARAM);
        return new ResponseEntity<>(new ApiResult<>(instance),instance.getStatus());
    }

    @ExceptionHandler(MissingServletRequestParameterException.class)
    public ResponseEntity<ApiResult<?>> handleMissingParamException() {
        clearCacheRTCMessage();
        ResponseCode instance = instance(COMMON_MISS_PARAMETER);
        return new ResponseEntity<>(new ApiResult<>(instance),instance.getStatus());
    }

    @ExceptionHandler(HttpMessageNotReadableException.class)
    public ResponseEntity<ApiResult<?>> handleHttpMessageNotReadableException(HttpMessageNotReadableException e) {
        clearCacheRTCMessage();
        log.warn("请求body格式错误: ", e);
        ResponseCode instance = instance(SYS_BODY_FORMAT_ERROR);
        return new ResponseEntity<>(new ApiResult<>(instance),instance.getStatus());
    }

    @ExceptionHandler(ConstraintViolationException.class)
    public ResponseEntity<ApiResult<?>> constraintViolationException(ConstraintViolationException e) {
        clearCacheRTCMessage();
        Set<ConstraintViolation<?>> constraintViolations = e.getConstraintViolations();
        ConstraintViolation<?> constraintViolation = constraintViolations.iterator().next();
        String field = constraintViolation.getPropertyPath().toString();
        field = field.substring(field.lastIndexOf(".") + 1);
        String error = constraintViolation.getMessage();
        ResponseCode instance = instance(error);
        if(instance == null) {
            instance = defaultValidate(error);
        }
        instance.setField(field);
        return new ResponseEntity<>(new ApiResult<>(instance),instance.getStatus());
    }

    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseEntity<ApiResult<?>> methodViolationException(MethodArgumentNotValidException e) {
        clearCacheRTCMessage();
        FieldError fieldError = e.getBindingResult().getFieldError();
        String field = null;
        String error = null;
        if(fieldError != null){
            field = fieldError.getField();
            error = fieldError.getDefaultMessage();
        }
        ResponseCode instance = defaultValidate(error);
        instance.setField(field);
        return new ResponseEntity<>(new ApiResult<>(instance),instance.getStatus());
    }

    @ExceptionHandler(BindException.class)
    public ResponseEntity<ApiResult<?>> bindException(BindException e) {
        clearCacheRTCMessage();
        FieldError fieldError = e.getBindingResult().getFieldError();
        String field = null;
        String chMsg = null;
        if(fieldError != null){
            field = fieldError.getField();
            chMsg = fieldError.getDefaultMessage();
        }
        ResponseCode instance = instance(COMMON_INVALID_PARAM);
        instance.setField(field);
        instance.setChMsg(chMsg);
        return new ResponseEntity<>(new ApiResult<>(instance),instance.getStatus());
    }

    @ExceptionHandler(CustomizeException.class)
    public ResponseEntity<ApiResult<?>> handleCommonException(CustomizeException e) {
        clearCacheRTCMessage();
        ApiResult<?> result = new ApiResult<>(e.getResponseCode(), e.getChMsg(), e.getEnMsg());
        return new ResponseEntity<>(result,e.getResponseCode().getStatus());
    }

    @ExceptionHandler(HttpRequestMethodNotSupportedException.class)
    public ResponseEntity<?> handleHttpRequestMethodNotSupportedException(HttpRequestMethodNotSupportedException e) {
        clearCacheRTCMessage();
        log.warn("不支持的请求方法: ", e);
        ResponseCode instance = instance(SYS_UNSUPPORTED_METHOD);
        return new ResponseEntity<>(new ApiResult<>(instance),instance.getStatus());
    }

    @ExceptionHandler(AccessDeniedException.class)
    public ResponseEntity<?> handleAccessDeniedException() {
        clearCacheRTCMessage();
        ResponseCode instance = instance(PERMISSION_DENIED);
        return new ResponseEntity<>(new ApiResult<>(instance),instance.getStatus());
    }

    @ExceptionHandler(MaxUploadSizeExceededException.class)
    public ResponseEntity<ApiResult<?>> maxUploadSizeExceededException() {
        clearCacheRTCMessage();
        ResponseCode instance = instance(SYS_FILE_SIZE_OUT_OF_GAUGE);
        return new ResponseEntity<>(new ApiResult<>(instance),instance.getStatus());
    }

    @ExceptionHandler(HttpMediaTypeNotSupportedException.class)
    public ResponseEntity<ApiResult<?>> httpMediaTypeNotSupportedException() {
        clearCacheRTCMessage();
        ResponseCode instance = instance(SYS_UNSUPPORTED_CONTENT_TYPE);
        return new ResponseEntity<>(new ApiResult<>(instance),instance.getStatus());
    }

    @ExceptionHandler(Exception.class)
    public ResponseEntity<ApiResult<?>> handleException(Exception e) {
        clearCacheRTCMessage();
        log.error("系统错误: ", e);
        ResponseCode instance = instance(SYS_INTERNAL_SERVER_ERROR);
        return new ResponseEntity<>(new ApiResult<>(instance),instance.getStatus());
    }

    /**
     * 出现异常 清空所有缓存消息
     */
    private void clearCacheRTCMessage() {
        try {
            messageRemindContext.clearCache();
            esJobContext.clearCache();
            esCustomerContext.clearCache();
        } catch (Throwable e) {
            log.error("clearCacheRTCMessage error ", e);
        }
    }
}
