package com.gosingapore.common.util;

import lombok.extern.slf4j.Slf4j;

import java.text.SimpleDateFormat;
import java.time.Instant;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author guomeng
 * @date 2020/02/12
 */
@Slf4j
public class DateUtils {

    /**
     * date2比date1多的天数
     *
     * @param date1 小日期
     * @param date2 大日期
     * @return 相差天数
     */
    public static int differentDays(Date date1, Date date2) {
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date1);

        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(date2);
        int day1 = cal1.get(Calendar.DAY_OF_YEAR);
        int day2 = cal2.get(Calendar.DAY_OF_YEAR);

        int year1 = cal1.get(Calendar.YEAR);
        int year2 = cal2.get(Calendar.YEAR);
        //同一年
        if (year1 != year2)
        {
            int timeDistance = 0;
            for (int i = year1; i < year2; i++) {
                //闰年
                if (i % 4 == 0 && i % 100 != 0 || i % 400 == 0)
                {
                    timeDistance += 366;
                } else    //不是闰年
                {
                    timeDistance += 365;
                }
            }

            return timeDistance + (day2 - day1);
        } else    //不同年
        {
            return day2 - day1;
        }
    }

    /**
     * 计算相差月数
     *
     * @param date1 <String>
     * @param date2 <String>
     * @return int
     */
    public static int getMonthSpace(Date date1, Date date2) {
        Calendar c1 = Calendar.getInstance();
        Calendar c2 = Calendar.getInstance();
        c1.setTime(date1);
        c2.setTime(date2);
        int year1 = c1.get(Calendar.YEAR);
        int year2 = c2.get(Calendar.YEAR);
        int month1 = c1.get(Calendar.MONTH);
        int month2 = c2.get(Calendar.MONTH);
        int day1 = c1.get(Calendar.DAY_OF_MONTH);
        int day2 = c2.get(Calendar.DAY_OF_MONTH);
        // 获取年的差值 
        int yearInterval = year1 - year2;
        // 如果 d1的 月-日 小于 d2的 月-日 那么 yearInterval-- 这样就得到了相差的年数
        boolean subtraction = month1 < month2 || month1 == month2 && day1 < day2;
        if (subtraction) {
            yearInterval--;
        }
        // 获取月数差值
        int monthInterval = (month1 + 12) - month2;
        if (day1 < day2) {
            monthInterval--;
        }
        monthInterval %= 12;
        return Math.abs(yearInterval * 12 + monthInterval);
    }

    /**
     * 根据指定的Format转化java.util.Date到String
     *
     * @param dt java.util.Date instance
     * @param sFmt Date format , DATE_FORMAT_DATEONLY or DATE_FORMAT_DATETIME
     * @return Date到String
     */
    public static String toString(Date dt, String sFmt) {
        if (dt == null || sFmt == null || "".equals(sFmt)) {
            return "";
        }
        return toString(dt, new SimpleDateFormat(sFmt));
    }

    /**
     * 利用指定SimpleDateFormat instance转换java.util.Date到String
     *
     * @param dt java.util.Date instance
     * @param formatter SimpleDateFormat Instance
     * @return Date到String
     */
    private static String toString(Date dt, SimpleDateFormat formatter) {
        String sRet = null;
        try {
            sRet = formatter.format(dt);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return sRet;
    }

    /**
     * 获取当前月初和月末日期
     *
     * @return map
     */
    public static Map<String, String> getFirstAndLastOfMonth() {
        Calendar caleFirst = Calendar.getInstance();
        caleFirst.add(Calendar.MONTH, 0);
        caleFirst.set(Calendar.DAY_OF_MONTH, 1);

        Calendar caleLast = Calendar.getInstance();
        caleLast.add(Calendar.MONTH, 1);
        caleLast.set(Calendar.DAY_OF_MONTH, 0);
        // 月初
        String first = caleFirst.getTime().toInstant().toString().substring(0, 10);
        // 月末
        String last = caleLast.getTime().toInstant().toString().substring(0, 10);
        Map<String, String> currentMonth = new HashMap<>(16);
        currentMonth.put("first", first);
        currentMonth.put("last", last);
        return currentMonth;
    }

    /**
     * 获取当天时间
     * 默认0点 + 增量秒
     * @param addSecond 增量 要增加的秒数
     * @return 当天时间
     */
    public static Date getTodayDate(Long addSecond){
        long now = Instant.now().getEpochSecond();
        long daySecond = 60 * 60 * 24;
        long earlyMorningTime = (now - (now + 8 * 3600) % daySecond + addSecond) * 1000 ;
        return new Date(earlyMorningTime);
    }

    /**
     * 获取当天 00:00:00 时间
     * @return 00:00:00
     */
    public static Date getTodayDate(){
        return getTodayDate(0L);
    }

    /**
     * 获取当天 23:59:59
     * @return 当天 23:59:59
     */
    public static Date getTodayEndDate(){
        long addSecond = (23*60+59) * 60L + 59;
        return getTodayDate(addSecond);
    }

    /**
     *
     * 获取前 n 天(过去的天数),并转化为指定格式
     * @param day 天数偏移量  (复数向左偏移,正数向右偏移)
     * @param sFmt 指定格式
     * @return 指定格式的过去时间
     */
    public static String getTodayOffset(Integer day,String sFmt){
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DATE,day);
        return toString(calendar.getTime(),sFmt);
    }
}
