package com.gosingapore.common.util;

import com.cloud.gs.hiring.base.util.ResponseCode;
import com.gosingapore.common.exception.CustomizeException;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.ReflectionUtils;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * 导出Excel工具类
 *
 * @author fangqiguang, copying
 */
public class ExportExcelUtil {

    public static <T> void exportOneSheetExcel(String fileName,String sheetName,List<String> headers,List<T> dataList,
                                               HttpServletResponse response){
        exportExcel(fileName, Collections.singletonList(sheetName),null,headers,Collections.singletonList(dataList),
                response,null);
    }
    /**
     * 导出多Sheet的Excel到HttpServletResponse流中
     *
     * <p>
     * 导出Excel格式：表头内容居中，字体略大于正文，颜色深灰色。正文文本类型对齐方式居左，
     * 数字类型对齐方式居右。仅有数据* 的单元格，有边框环绕，实体类的属性顺序即为表头顺序
     * </p>
     * @param fileName          另存为文件名
     * @param sheetNames        工作簿中的多张Sheet工作表的名称列表
     * @param titleName         表格的标题名称（没有标题，则传null）
     * @param headers           表头列表 (添加功能,当要导出的字段名为""时,则代表这个字段不用导出)
     * @param dataLists         要导出的数据源
     * @param response          Http响应
     * @param pattern           时间类型数据的格式,默认UTC格式
     */
    public static <T> void exportExcel(String fileName, List<String> sheetNames, String titleName, List<String> headers,
                                       List<List<T>> dataLists, HttpServletResponse response, String pattern) {
        XSSFWorkbook wb = exportAllExcel(sheetNames, titleName, headers, dataLists, pattern);
        setResponseHeader(response, replaceSpecStr(fileName));
        ServletOutputStream out = null;
        try {
            out = response.getOutputStream();
            wb.write(out);
        } catch (IOException e) {
            e.printStackTrace();
            throw new CustomizeException(ResponseCode.SYS_INTERNAL_SERVER_ERROR, "导出Excel失败", "Failed to export Excel");
        } finally {
            try {
                assert out != null;
                out.flush();
                out.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 设置响应的类型、编码和文件名称
     *
     * @param response 参数
     * @param fileName 文件名
     */
    public static void setResponseHeader(HttpServletResponse response, String fileName) {
        try {
            response.reset();
            // 设置生成的文件类型
            response.setContentType("application/msexcel");
            // 设置文件头编码方式和文件名
            response.setCharacterEncoding("UTF-8");
            // 在浏览器中测试生效，postman中文件名为response,无法修改
            response.setHeader("Content-disposition", "attachment;filename="
                    .concat(String.valueOf(URLEncoder.encode(replaceSpecStr(fileName) + ".xlsx", "UTF-8"))));
            // 此设置，可保证web端可以取到文件名
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // 正则判断是否为数值
    private static final Pattern P = Pattern.compile("^\\d+(\\.\\d+)?$");
    /**
     * 多Sheet导出实现
     *
     * @param sheetNames        sheetNames
     * @param titleName         titleName
     * @param headers           headers
     * @param dataLists         dataLists
     * @param pattern           pattern
     * @return T
     */
    private static <T> XSSFWorkbook exportAllExcel(List<String> sheetNames, String titleName, List<String> headers,
                                                   List<List<T>> dataLists, String pattern) {
        List<String> trueHeaders = headers.stream().filter(header -> !"".equals(header)).collect(Collectors.toList());
        // 创建一个工作薄
        XSSFWorkbook workbook = new XSSFWorkbook();
        for (int i = 0; i < dataLists.size(); i++) {
            // 创建一个工作表
            XSSFSheet sheet = workbook.createSheet(replaceSpecStr(sheetNames.get(i)));
            setSheetStyle(workbook,sheet,titleName,trueHeaders.size());
            // 创建表头样式
            XSSFCellStyle headersStyle = createSheetHeaderStyle(workbook);
            // 创建数据样式
            XSSFCellStyle dataSetStyle = createSheetDateStyle(workbook);
            int index = ObjectUtils.isEmpty(titleName) ? 0 : 1;
            // 创建表头并设置样式
            XSSFRow row = sheet.createRow(index);
            for (short j = 0; j < trueHeaders.size(); j++) {
                XSSFCell cell = row.createCell(j);
                cell.setCellStyle(headersStyle);
                XSSFRichTextString text = new XSSFRichTextString(trueHeaders.get(j));
                cell.setCellValue(text);
            }
            // 导出正文数据，并设置其样式
            for (T entity : dataLists.get(i)) {
                index++;
                row = sheet.createRow(index);
                // 利用反射，根据实体类属性的先后顺序，动态调用其getXxx()方法，得到属性值
                List<Field> fields = new ArrayList<>(Arrays.asList(entity.getClass().getDeclaredFields()));
                // 获取父类属性
                if(!ObjectUtils.isEmpty(entity.getClass().getSuperclass())){
                    fields.addAll(Arrays.asList(entity.getClass().getSuperclass().getDeclaredFields()));
                }
                if (headers.size() != fields.size()){
                    throw new CustomizeException(ResponseCode.COMMON_FAIL_REQUEST,
                            "设置表格标题数量与实际数据数量不符","");
                }
                int dataIndex = 0;
                for (short k = 0; k < fields.size(); k++) {
                    // 添加功能,当要导出的字段名为""时,则代表这个字段不用导出
                    if (ObjectUtils.isEmpty(headers.get(k))){
                        continue;
                    }
                    XSSFCell cell = row.createCell(dataIndex);
                    Field field = fields.get(k);
                    String fieldName = field.getName();
                    String getMethodName = "get" + fieldName.substring(0, 1).toUpperCase() + fieldName.substring(1);
                    try {
                        @SuppressWarnings("rawtypes")
                        Class entityClass = entity.getClass();
                        @SuppressWarnings("unchecked")
                        Method getMethod = entityClass.getMethod(getMethodName);
                        Object value = getMethod.invoke(entity);
                        String textValue;
                        // 如果是时间类型,格式化
                        if (value instanceof Date) {
                            Date date = (Date) value;
                            pattern = ObjectUtils.isEmpty(pattern) ? "yyyy-MM-dd'T'HH:mm:ss'Z'" : pattern;
                            SimpleDateFormat sdf = new SimpleDateFormat(pattern);
                            textValue = sdf.format(date);
                        } else {
                            textValue = ObjectUtils.isEmpty(value) ? "" : String.valueOf(value);
                        }
                        // 边框环绕
                        cell.setCellStyle(dataSetStyle);
                        if (!"".equals(textValue)) {
                            Matcher matcher = P.matcher(textValue);
                            if (matcher.matches()) {
                                // 是数字当作double处理，整型也不会补充小数点
                                cell.setCellValue(Double.parseDouble(textValue));
                            } else {
                                // 不是数字类型作为文本输出
                                cell.setCellValue(textValue);
                            }
                        }
                    } catch (SecurityException | NoSuchMethodException | IllegalArgumentException | IllegalAccessException | InvocationTargetException e) {
                        e.printStackTrace();
                        throw new CustomizeException(ResponseCode.SYS_INTERNAL_SERVER_ERROR, "导出失败！", "Export failed!");
                    }
                    dataIndex++;
                }
            }
        }
        return workbook;
    }

    /**
     * 设置 工作簿 sheet 样式
     */
    private static void setSheetStyle(XSSFWorkbook workbook, XSSFSheet sheet,String titleName,Integer headSize){
        // 设置单元格列宽度为16个字节
        sheet.setDefaultColumnWidth((short) 16);
        if (titleName != null && !"".equals(titleName)) {
            XSSFCellStyle titleStyle = workbook.createCellStyle();
            // 将首行合并居中作为标题栏
            sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, headSize - 1));
            XSSFFont titleFont = workbook.createFont();
            // 设置标题字体大小
            titleFont.setFontHeightInPoints((short) 20);
            // 设置标题字体样式
            titleStyle.setFont(titleFont);
            // 创建标题行并设置样式
            XSSFRow titleRow = sheet.createRow(0);
            XSSFCell titleCell = titleRow.createCell(0);
            titleCell.setCellStyle(titleStyle);
            titleCell.setCellValue(titleName);
        }
    }

    /**
     * 创建 表头样式
     * @param workbook 工作簿
     * @return 表头样式
     */
    private static XSSFCellStyle createSheetHeaderStyle(XSSFWorkbook workbook){
        // 创建表头样式
        XSSFCellStyle headersStyle = workbook.createCellStyle();
        headersStyle.setBorderTop(BorderStyle.THIN);
        headersStyle.setBorderBottom(BorderStyle.THIN);
        headersStyle.setBorderLeft(BorderStyle.THIN);
        headersStyle.setBorderRight(BorderStyle.THIN);
        // 表头内容对齐方式：居中
        headersStyle.setAlignment(HorizontalAlignment.CENTER);
        XSSFFont headersFont = workbook.createFont();
        // 设置字体格式
        headersFont.setColor(new XSSFColor(java.awt.Color.DARK_GRAY));
        headersFont.setFontHeightInPoints((short) 14);
        // 表头样式应用生效
        headersStyle.setFont(headersFont);
        return headersStyle;
    }

    /**
     * 创建 数据格式
     * @param workbook 工作簿
     * @return 数据格式
     */
    private static XSSFCellStyle createSheetDateStyle(XSSFWorkbook workbook){
        // 创建数据样式
        XSSFCellStyle dataSetStyle = workbook.createCellStyle();
        // 正文单元格边框样式
        dataSetStyle.setBorderBottom(BorderStyle.THIN);
        dataSetStyle.setBorderRight(BorderStyle.THIN);
        dataSetStyle.setBorderLeft(BorderStyle.THIN);
        // 数据内容对齐方式：居左
        dataSetStyle.setAlignment(HorizontalAlignment.CENTER_SELECTION);
        XSSFFont dataSetFont = workbook.createFont();
        // 正文字体颜色
        dataSetFont.setColor(new XSSFColor(java.awt.Color.DARK_GRAY));
        // 为正文设置样式
        dataSetStyle.setFont(dataSetFont);
        return dataSetStyle;
    }

    /**
     * 正则替换所有特殊字符
     *
     * @param orgStr orgStr
     * @return String
     */
    public static String replaceSpecStr(String orgStr) {
        if (null != orgStr && !"".equals(orgStr.trim())) {
            String regEx = "[\\\\|:/\"<>?*\\[\\] ]";
            Pattern p = Pattern.compile(regEx);
            Matcher m = p.matcher(orgStr);
            return m.replaceAll("_");
        }
        return null;
    }

}