package com.gosingapore.common.util;

import com.gosingapore.common.model.dto.BaseTreeDTO;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author copying
 * @version 3.0
 * @date 2021/3/19 9:59
 */
public class ListUtils {

    /**
     * 将指定格式的 list 集合 转化为 树状结构
     * 集合中的对象 必须 继承 {@link BaseTreeDTO}
     * <p>
     *      并重写 自己的set方法 将获取到的 自己本身的节点id和父节点id 复制到
     *      @see BaseTreeDTO#setNoteId(Integer)
     *      @see BaseTreeDTO#setParentId(Integer)
     * </p>
     * @param resourceList 源数据集合
     * @param rootId 指定顶级节点id
     * @param <E> 指定类型
     * @return 集合转化为的树状结构
     */
    public static <E extends BaseTreeDTO<E>> List<E> getTree(List<E> resourceList, Integer rootId){
        if (CollectionUtils.isEmpty(resourceList)){
            return new ArrayList<>(0);
        }
        Map<Integer,List<E>> resourceMap = resourceList.stream().collect(Collectors.groupingBy(E::getParentId));
         if (ObjectUtils.isEmpty(resourceMap)){
            return new ArrayList<>(0);
        }
        // 获取根节点集合
        List<E> rootNotes = resourceMap.get(rootId);
        if (CollectionUtils.isEmpty(rootNotes)){
            resourceMap = resourceList.stream().collect(Collectors.groupingBy(E::getNoteId));
            return resourceMap.getOrDefault(rootId,new ArrayList<>(0));
        }
        assemblyTree(resourceMap,rootNotes);
        return rootNotes;
    }

    private static <E extends BaseTreeDTO<E>> void assemblyTree(Map<Integer, List<E>> resource, List<E> rootList){
        for (E e : rootList) {
            List<E> childList = resource.get(e.getNoteId());
            if (CollectionUtils.isEmpty(childList)){
                continue;
            }
            assemblyTree(resource,childList);
            e.setChildList(childList);
        }
    }


    /**
     * 根据 树的节点id 获取 直到根节点的一条分支
     *
     * @param treeSource 节点 所在的 树
     * @param noteId 节点id
     * @param <E> 节点类型 继承了 {@link BaseTreeDTO}
     * @return 节点所在的一条分支
     */
    public static <E extends BaseTreeDTO<E>> E getOneBranch(List<E> treeSource, Integer noteId){
        if (CollectionUtils.isEmpty(treeSource)){
            return null;
        }
        for (E e:treeSource){
            if (e.getNoteId().equals(noteId)){
                return e;
            }
            if (!CollectionUtils.isEmpty(e.getChildList())){
                E note = getParentNote(e.getChildList(),noteId,e);
                if (ObjectUtils.isEmpty(note)){
                    continue;
                }
                return note;
            }
        }
        return null;
    }
    private static <E extends BaseTreeDTO<E>> E getParentNote(List<E> treeSource,Integer noteId,E parent){
        for (E e:treeSource){
            if (e.getNoteId().equals(noteId)){
                parent.setChildList(Collections.singletonList(e));
                return parent;
            }
            if (!CollectionUtils.isEmpty(e.getChildList())){
                E note = getParentNote(e.getChildList(),noteId,e);
                if (ObjectUtils.isEmpty(note)){
                    continue;
                }
                parent.setChildList(Collections.singletonList(note));
                return parent;
            }
        }
        return null;
    }

    /**
     * 拼接 List<String>
     * @param stringList List<String>
     * @param sign 拼接符
     * @return String
     */
    public static String spliceStringList(String sign,List<String> stringList){
        StringBuffer stringBuffer = new StringBuffer();
        if (stringList == null || stringList.isEmpty()){
            stringBuffer.append("无");
        }else {
            stringBuffer.append(stringList.remove(0));
            stringList.forEach(p->{
                stringBuffer.append(sign);
                stringBuffer.append(p);
            });
        }
        return String.valueOf(stringBuffer);
    }

    /**
     * 拼接 List<List<String>>
     * @param sign 拼接符
     * @param noteSign 节点拼接符
     * @return String
     */
    public static String spliceStringLists(String sign, String noteSign, List<List<String>> stringList){
        StringBuffer string = new StringBuffer();
        for (int i=0; i < stringList.size(); i++){
            List<String> list = stringList.get(i);
            if (i > 0){
                string.append(noteSign);
            }
            string.append(spliceStringList(sign,list));
        }
        return String.valueOf(string);
    }
}
