package com.gosingapore.common.util;

import com.alibaba.fastjson.JSONArray;
import com.cloud.gs.hiring.base.util.ResponseCode;
import com.gosingapore.common.constants.RabbitMqConstants;
import com.gosingapore.common.enums.dict.Dict127MsgRemindClassEnum;
import com.gosingapore.common.enums.dict.Dict44MsgRemindScopeEnum;
import com.gosingapore.common.exception.CustomizeException;
import com.gosingapore.common.model.domain.message.ErpMessage;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 消息推送上下文
 *
 * @author shuyan
 */
@Slf4j
@Component
public class MessageRemindContext {
    /**
     * 线程安全
     */
    private static final ThreadLocal<ContextCache> CONTEXT = ThreadLocal.withInitial(ContextCache::new);

    @Resource
    private RabbitTemplate rabbitTemplate;
    /**
     * 给自营用户推送
     * @param userId 用户id
     * @param msg    消息内容
     */
    public void selfCachePut(Integer userId, ErpMessage msg) {
        if (ObjectUtils.isEmpty(userId)){
            return;
        }
        msg.setUserId(userId);
        msg.setScopeValue(Dict44MsgRemindScopeEnum.PERSONAL.getValue());
        checkParam(msg);
        selfCachePut(msg);
    }
    /**
     * 给自营用户推送
     * @param msg    消息内容
     */
    public void selfCachePut(ErpMessage msg) {
        if (ObjectUtils.isEmpty(msg)){
            return;
        }
        cachePut(Collections.singletonList(msg));
    }

    /**
     * 给自营用户推送 批量
     * @param msg    消息内容
     */
    public void selfCachePut(List<ErpMessage> msg) {
        if (CollectionUtils.isEmpty(msg)){
            return;
        }
        cachePut(msg);
    }

    /**
     * selfCachePut 给自营用户推送 批量
     * @param userIds userIds
     * @param message message
     */
    public void selfCachePut(List<Integer> userIds, ErpMessage message){
        if (CollectionUtils.isEmpty(userIds) || ObjectUtils.isEmpty(message)){
            return;
        }
        message.setScopeValue(Dict44MsgRemindScopeEnum.PERSONAL.getValue());
        List<ErpMessage> messageList = new ArrayList<>(userIds.size());
        userIds.forEach(userId->{
            ErpMessage msg = BeanUtil.copyProperties(message,ErpMessage.class);
            msg.setUserId(userId);
            messageList.add(msg);
        });
        selfCachePut(messageList);
    }

    private void cachePut(List<ErpMessage> msg) {
        ContextCache contextCache = CONTEXT.get();
        List<ErpMessage> messages = contextCache.getMessages();
        messages.addAll(msg);
    }

    /**
     * 检查参数
     * @param message message
     */
    private void checkParam(ErpMessage message) {
        if (Dict127MsgRemindClassEnum.instanceOf(message.getClassValue()) == null) {
            log.info("GsMessageRemind.classValue 不在允许值范围内；message：{}", message);
            throw new CustomizeException(ResponseCode.COMMON_NOT_IN_ALLOWED_SCOPE, "GsMessageRemind.classValue 不在允许值范围内", "GsMessageRemind.classValue not in allowed scope");
        }
        if (Dict44MsgRemindScopeEnum.instanceOf(message.getScopeValue()) == null) {
            log.info("GsMessageRemind.scopeValue 不在允许值范围内；message：{}", message);
            throw new CustomizeException(ResponseCode.COMMON_NOT_IN_ALLOWED_SCOPE, "GsMessageRemind.scopeValue 不在允许值范围内", "GsMessageRemind.scopeValue not in allowed scope");
        }
    }

    public void sendCache() {
        List<ErpMessage> messages = CONTEXT.get().getMessages();
        if (messages.size() > 0) {
            rabbitTemplate.convertAndSend(RabbitMqConstants.ERP_MESSAGE_QUEUE, JSONArray.toJSONString(messages));
            log.info("成功发送 {} 条erp消息", messages.size());
        }
    }


    public void clearCache() {
        ContextCache contextCache = CONTEXT.get();
        contextCache.getMessages().clear();
    }

    public void removeContext() {
        CONTEXT.remove();
    }

    @Data
    private static class ContextCache {
        /**
         * 缓存所有消息
         */
        private List<ErpMessage> messages = new ArrayList<>();
    }
}

