package com.gosingapore.common.util;

import com.gosingapore.common.constants.StringContains;
import com.gosingapore.common.enums.MobileCountryEnum;
import com.gosingapore.common.enums.dict.Dict50AreaCodeEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.ObjectUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.regex.Pattern;

/**
 * 手机号合法性检查工具类
 *
 * @author ZhangJinyu
 * @since 2020-09-16
 */
public class MobileHandlerUtil {
    private MobileHandlerUtil() {
    }

    private static final String AREA_CODE_FIRST = "+";
    /**
     * 校验手机号是否合法
     *
     * @param isMixing          是否是混合的.true:手机号和座机号 false:手机号
     * @param mobileCountryCode 手机号国家代码
     * @param mobile            手机号
     * @return 是否合法 true:合法 false:不合法
     */
    public static boolean checkMobile(boolean isMixing, String mobileCountryCode, String mobile) {
        MobileCountryEnum mobileCountryEnum = MobileCountryEnum.getByCode(mobileCountryCode);
        Pattern mobilePattern = mobileCountryEnum.getMobilePattern();
        boolean isCorrect = false;
        if (isMixing) {
            Pattern mixingPattern = mobileCountryEnum.getFixedTelephonePattern();
            if (!ObjectUtils.isEmpty(mixingPattern)) {
                isCorrect = mixingPattern.matcher(mobile).matches();
            }
        }
        if (!isCorrect) {
            isCorrect = !ObjectUtils.isEmpty(mobilePattern) && mobilePattern.matcher(mobile).matches();
        }
        return isCorrect;
    }

    /**
     * 获取手机号
     *
     * @param mobileCountryCode 手机号国家代码
     * @param mobile            手机号
     * @return Optional<MobileCountryEnum.Mobile> get()可能为null,请使用isPresent()校验后处理,防止NPE出现
     */
    public static Optional<MobileCountryEnum.Mobile> getMobile(String mobileCountryCode, String mobile) {
        MobileCountryEnum mobileCountryEnum = MobileCountryEnum.getByCode(mobileCountryCode);
        if (checkMobile(false, mobileCountryCode, mobile)) {
            if (mobileCountryEnum.equals(MobileCountryEnum.SINGAPORE)) {
                if (StringUtils.startsWith(mobile, StringContains.ZERO)) {
                    mobile = mobile.replaceFirst(mobile, "");
                }
            }
            return Optional.of(new MobileCountryEnum.Mobile(mobileCountryCode, mobile));
        }
        return Optional.empty();
    }

    /**
     * // 截取手机号中的地区编码（+86之类的东西）
     * 取得纯净的手机号
     * @param mobile 手机号
     * @return 地区编码标识 key:areaCodeValue {@link Dict50AreaCodeEnum} ,手机号 key:mobile
     */
    public static Map<String,String> getTidyMobile(String mobile){
        if(mobile!= null && mobile.startsWith(AREA_CODE_FIRST)){
            String areaCode = mobile.substring(1, 3);
            Dict50AreaCodeEnum codeByCode = Dict50AreaCodeEnum.getCodeByCode(areaCode);
            if(codeByCode != null){
                Map<String,String> map = new HashMap<>(2);
                map.put("areaCodeValue",String.valueOf(codeByCode.getValue()));
                map.put("mobile",mobile.substring(3));
                return map;
            }
        }
        return null;
    }
}
