package com.gosingapore.common.util;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.model.GetObjectRequest;
import com.aliyun.oss.model.OSSObject;
import com.aliyun.oss.model.OSSObjectSummary;
import com.aliyun.oss.model.ObjectListing;
import com.gosingapore.common.properties.AliCloudOssProperties;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

/**
 * @author C.K
 * @date 2019/5/9 15:06
 * @version 1.0
 */
@Slf4j
@Component
@SuppressWarnings("unused")
public class OssUtil {
    @Resource
    private AliCloudOssProperties aliCloudOssProperties;

    /**
     * @return OSSClient oss客户端
     */
    private OSS getOssClient() {
        String endpoint = aliCloudOssProperties.getEndpoint();
        String accessKeyId = aliCloudOssProperties.getAccessKey().getAccessKeyId();
        String accessKeySecret = aliCloudOssProperties.getAccessKey().getAccessKeySecret();
        return new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
    }

    /**
     * 通过网络流上传文件
     * @param url 			URL
     * @param objectName 	图片/文件名
     * @return void 		返回类型
     */
    public String uploadByNetworkStream(URL url, String objectName) {
        OSS ossClient = getOssClient();
        try {
            InputStream inputStream = url.openStream();
            ossClient.putObject(aliCloudOssProperties.getBucket(), objectName, inputStream);
            ossClient.shutdown();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            ossClient.shutdown();
        }
        return aliCloudOssProperties.getUrl() + objectName;
    }

    /**
     *
     * 通过输入流上传文件
     * @param inputStream 	输入流
     * @param objectName 	上传文件目录和（包括文件名） 例如“test/a.jpg”
     * @return void 		返回类型
     */
    public String uploadByInputStream(InputStream inputStream, String objectName) {
        OSS ossClient = getOssClient();
        try {
            ossClient.putObject(aliCloudOssProperties.getBucket(), objectName, inputStream);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            ossClient.shutdown();
        }
        return aliCloudOssProperties.getUrl() + objectName;
    }

    /**
     *
     * 通过file上传文件
     * @param file 			上传的文件
     * @param objectName 	上传文件目录和（包括文件名） 例如“test/a.jpg”
     * @return void 		返回类型
     */
    public String uploadByFile(File file, String objectName) {
        OSS ossClient = getOssClient();
        try {
            ossClient.putObject(aliCloudOssProperties.getBucket(), objectName, file);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            ossClient.shutdown();
        }
        return aliCloudOssProperties.getUrl() + objectName;
    }


    /**
     * 下载文件到本地
     * @param objectName    oss文件名称，eg：123.png
     * @param baseUrl       本地路径    c:\\download
     * @return 资源地址
     */
    public String downFile(String objectName, String baseUrl){
        OSS ossClient = getOssClient();
        try {
            ossClient.getObject(new GetObjectRequest(aliCloudOssProperties.getBucket(), objectName), new File(baseUrl+"\\"+objectName));
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            ossClient.shutdown();
        }
        return aliCloudOssProperties.getUrl() + objectName;

    }


    /**
     *
     * 根据key删除oss服务器上的文件
     * @param key    		文件路径/名称，例如“test/a.txt”
     */
    public void deleteFile(String key) {
        getOssClient().deleteObject(aliCloudOssProperties.getBucket(), key);
    }

    /**
     *
     * 根据key获取服务器上的文件的输入流
     * @param key 			文件路径和名称
     * @return InputStream 	文件输入流
     */
    public InputStream getInputStreamByOss(String key) {
        OSS ossClient = getOssClient();
        InputStream content = null;
        try {
            OSSObject ossObj = ossClient.getObject(aliCloudOssProperties.getBucket(), key);
            content = ossObj.getObjectContent();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return content;
    }

    /**
     *
     * 查询某个bucket里面的所有文件
     * @return List<String>  文件路径和大小集合
     */
    public List<String> queryAllObject() {
        OSS ossClient = getOssClient();
        List<String> results = new ArrayList<>();
        try {
            // ossClient.listObjects返回ObjectListing实例，包含此次listObject请求的返回结果。
            ObjectListing objectListing = ossClient.listObjects(aliCloudOssProperties.getBucket());
            // objectListing.getObjectSummaries获取所有文件的描述信息。
            for (OSSObjectSummary objectSummary : objectListing.getObjectSummaries()) {
                results.add(" - " + objectSummary.getKey() + "  " + "(size = " + objectSummary.getSize() + ")");
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            ossClient.shutdown();
        }
        return results;
    }

}
