package com.gosingapore.common.util;

import com.google.zxing.*;
import com.google.zxing.client.j2se.BufferedImageLuminanceSource;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.common.HybridBinarizer;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;

import javax.imageio.ImageIO;
import javax.imageio.stream.ImageOutputStream;
import java.awt.*;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.Date;
import java.util.Hashtable;

/**
 * @author zbw
 * @date 2019/10/21/0219:02
 **/
public class QrCodeUtil {
    private static final String CHARSET = "utf-8";
    private static final String FORMAT_NAME = "JPG";
    /**
     * 二维码尺寸，宽度和高度均是300
     */
    private static final int QR_CODE_SIZE = 300;
    /**
     * LOGO宽度
     */
    private static final int WIDTH = 80;
    /**
     * LOGO高度
     */
    private static final int HEIGHT = 80;

    /**
     * 生成二维码的方法
     *
     * @param content      目标url
     * @param imgPath      logo图片地址
     * @param needCompress 是否压缩logo
     * @return 二维码图片
     * @throws Exception 异常
     */
    private static BufferedImage createImage(String content, String imgPath, boolean needCompress) throws Exception {
        Hashtable<EncodeHintType,Object> hints = new Hashtable<>();
        //指定二维码的纠错等级为高级
        hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.H);
        //指定字符编码为“utf-8”
        hints.put(EncodeHintType.CHARACTER_SET, CHARSET);
        //设置图片的边距
        hints.put(EncodeHintType.MARGIN, 1);
        //参数1：内容，目标url,参数2：固定写法，参数3：二维码的宽度，参数4：二维码的高度，参数5：二维码属性设置
        BitMatrix bitMatrix = new MultiFormatWriter().encode(content, BarcodeFormat.QR_CODE, QR_CODE_SIZE, QR_CODE_SIZE, hints);
        //二维码的宽度
        int width = bitMatrix.getWidth();
        //二维码的高度
        int height = bitMatrix.getHeight();
        //生成的二维码image
        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                image.setRGB(x, y, bitMatrix.get(x, y) ? 0xFF000000 : 0xFFFFFFFF);
            }
        }
        //logo图片地址为null或空时
        if (imgPath == null || "".equals(imgPath)) {
            //只返回二维码图片，无中间的logo
            return image;
        }
        // 插入logo图片  参数1：二维码图片，参数2：logo图片地址，参数3：压缩图片
        QrCodeUtil.insertImage(image, imgPath, needCompress);
        return image;
    }

    /**
     * 插入logo图片
     *
     * @param source       二维码图片
     * @param imgPath      LOGO图片地址
     * @param needCompress 是否压缩
     * @throws Exception 异常
     */
    private static void insertImage(BufferedImage source, String imgPath, boolean needCompress) throws Exception {
        //logo图片地址放入文件
        File file = new File(imgPath);
        if (!file.exists()) {
            System.err.println("" + imgPath + " 该文件不存在！");
            return;
        }
        Image src = ImageIO.read(new File(imgPath));
        int width = src.getWidth(null);
        int height = src.getHeight(null);
        // 压缩LOGO
        if (needCompress) {
            //如果上传logo宽 >60
            if (width > WIDTH) {
                width = WIDTH;
            }
            //如果上传logo高>60
            if (height > HEIGHT) {
                height = HEIGHT;
            }
            Image image = src.getScaledInstance(width, height, Image.SCALE_SMOOTH);
            BufferedImage tag = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
            Graphics g = tag.getGraphics();
            // 绘制缩小后的图
            g.drawImage(image, 0, 0, null);
            g.dispose();
            src = image;
        }
        // 插入LOGO
        Graphics2D graph = source.createGraphics();
        int x = (QR_CODE_SIZE - width) / 2;
        int y = (QR_CODE_SIZE - height) / 2;
        graph.drawImage(src, x, y, width, height, null);
        Shape shape = new RoundRectangle2D.Float(x, y, width, width, 6, 6);
        graph.setStroke(new BasicStroke(3f));
        graph.draw(shape);
        graph.dispose();
    }

    /**
     * 生成二维码(内嵌LOGO)
     *
     * @param content      内容
     * @param imgPath      logo地址
     * @param destPath     存放目录
     * @param needCompress 是否压缩logo
     * @param fileName     文件名
     * @throws Exception 异常
     */
    public static void encode(String content, String imgPath, String destPath, boolean needCompress, String fileName) throws Exception {
        //生成二维码
        BufferedImage image = QrCodeUtil.createImage(content, imgPath, needCompress);
        mkdirs(destPath);
        String file = fileName + ".jpg";
        ImageIO.write(image, FORMAT_NAME, new File(destPath + "/" + file));
    }

    /**
     * 获取二维码流数据
     * @author zbw
     * @date 16:20 2019/10/25/025
     **/
    public static InputStream encodeForInputStream(String content, String imgPath, boolean needCompress) throws Exception {
        //生成二维码
        BufferedImage imgBar = QrCodeUtil.createImage(content, imgPath, needCompress);
        ByteArrayOutputStream bs = new ByteArrayOutputStream();
        ImageOutputStream imOut = ImageIO.createImageOutputStream(bs);
        ImageIO.write(imgBar, "jpg", imOut);
        return new ByteArrayInputStream(bs.toByteArray());
    }

    /**
     * 当文件夹不存在时，mkdirs会自动创建多层目录，区别于mkdir．(mkdir如果父目录不存在则会抛出异常)
     *
     * @param destPath 存放目录
     */
    private static void mkdirs(String destPath) {
        File file = new File(destPath);
        if (!file.exists() && !file.isDirectory()) {
            //noinspection ResultOfMethodCallIgnored
            file.mkdirs();
        }
    }

    /**
     * 生成二维码(内嵌LOGO),没有压缩
     *
     * @param content  内容
     * @param imgPath  LOGO地址
     * @param destPath 存储地址
     * @param fileName 文件名
     * @throws Exception 异常
     */
    public static void encode(String content, String imgPath, String destPath, String fileName) throws Exception {
        QrCodeUtil.encode(content, imgPath, destPath, false, fileName);
    }

    /**
     * 生成二维码，无内嵌logo
     *
     * @param content      内容
     * @param destPath     存储地址
     * @param needCompress 是否压缩LOGO
     * @param fileName     文件名
     * @throws Exception 异常
     */
    public static void encode(String content, String destPath, boolean needCompress, String fileName) throws Exception {
        QrCodeUtil.encode(content, null, destPath, needCompress, fileName);
    }

    /**
     * 生成二维码
     *
     * @param content  内容
     * @param destPath 存储地址
     * @param fileName 文件名
     * @throws Exception 异常
     */
    public static void encode(String content, String destPath, String fileName) throws Exception {
        QrCodeUtil.encode(content, null, destPath, false, fileName);
    }

    /**
     * 生成二维码(内嵌LOGO)
     *
     * @param content      内容
     * @param imgPath      LOGO地址
     * @param output       输出流
     * @param needCompress 是否压缩LOGO
     * @throws Exception 异常
     */
    public static void encode(String content, String imgPath, OutputStream output, boolean needCompress) throws Exception {
        BufferedImage image = QrCodeUtil.createImage(content, imgPath, needCompress);
        ImageIO.write(image, FORMAT_NAME, output);
    }

    /**
     * 生成二维码
     *
     * @param content 内容
     * @param output  输出流
     * @throws Exception 异常
     */
    public static void encode(String content, OutputStream output) throws Exception {
        QrCodeUtil.encode(content, null, output, false);
    }

    /**
     * 解析二维码
     * 按文件参数解析
     *
     * @param file 二维码图片
     * @return 二维码文本内容
     * @throws Exception 异常
     */
    public static String decode(File file) throws Exception {
        BufferedImage image;
        image = ImageIO.read(file);
        if (image == null) {
            return null;
        }
        BufferedImageLuminanceSource source = new BufferedImageLuminanceSource(image);
        BinaryBitmap bitmap = new BinaryBitmap(new HybridBinarizer(source));
        Result result;
        Hashtable<DecodeHintType,Object> hints = new Hashtable<>();
        //指定字符编码为“utf-8”
        hints.put(DecodeHintType.CHARACTER_SET, CHARSET);
        result = new MultiFormatReader().decode(bitmap, hints);
        return result.getText();
    }

    /**
     * 解析二维码
     * 按二维码图片地址解析
     *
     * @param path 二维码图片地址
     * @return 不是二维码的内容返回null, 是二维码直接返回识别的结果
     * @throws Exception 异常
     */
    public static String decode(String path) throws Exception {
        return QrCodeUtil.decode(new File(path));
    }

    public static String renameTheQrCode(String inviteCode, String folderName){
        String nowTime = DateUtils.toString(new Date(), "yyyyMMddHHmmss");
        return folderName + "/" + inviteCode + "_" + nowTime + ".jpg";
    }
}
