package com.gosingapore.common.util;

import com.cloud.gs.hiring.base.util.ResponseCode;
import com.gosingapore.common.exception.CustomizeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.concurrent.TimeUnit;

/**
 * @author shuyan
 * @date 2020/5/14 10:31
 */
@Slf4j
@Component
public class RedisLockUtil {
    private static final String PRE_SYNC_LOCK_EXPIRE_KEY = "EXPIRE_KEY_";
    @Resource
    private RedisTemplate<String,String> redisTemplate;

    /**
     * 获取阻塞锁
     * @param lockKey 锁名称
     * @param lockValue 锁值，如果调用该方法时，已经获取了这把锁，则将父级锁值传入，用以支持重入锁，防止死锁
     */
    public String getBlockingQueueLock(String lockKey, String lockValue){
        if(lockValue == null) {
            lockValue = UUIDUtils.get();
        }
        redisTemplate.opsForList().rightPush(lockKey,lockValue);
        String firstLockValue;
        while (!lockValue.equals(firstLockValue = redisTemplate.opsForList().index(lockKey, 0))){
            if(lockValue.equals(redisTemplate.opsForList().index(lockKey, 1)) && !getNoneBlockingLockStatus(PRE_SYNC_LOCK_EXPIRE_KEY + lockKey)){
                unlockBlockingQueueLock(lockKey,firstLockValue);
            }else{
                try {
                    Thread.sleep(100);
                }catch (Exception ignore){}
            }
        }
        getNoneBlockingLock(PRE_SYNC_LOCK_EXPIRE_KEY + lockKey);
        return lockValue;
    }

    /**
     * 释放阻塞锁
     * @param lockKey 锁名称
     * @param lockValue 锁值
     */
    public void unlockBlockingQueueLock(String lockKey, String lockValue){
        if(lockValue != null && lockValue.equals(redisTemplate.opsForList().index(lockKey, 0))){
            unlockNoneBlockingLock(PRE_SYNC_LOCK_EXPIRE_KEY + lockKey);
            redisTemplate.opsForList().leftPop(lockKey);
            return;
        }
        // 释放不成功，未在获取到锁后的有效期内释放锁
        throw new CustomizeException(ResponseCode.SYS_INTERNAL_SERVER_ERROR,"释放分布式锁失败","Failed to release distributed lock");
    }

    /**
     * 获取非阻塞锁
     * @param lockKey 锁名称
     * @return 是否获取成功
     */
    public Boolean getNoneBlockingLock(String lockKey){
        Boolean ifAbsent = redisTemplate.opsForValue().setIfAbsent(lockKey, "",5,TimeUnit.SECONDS);
        return ifAbsent != null && ifAbsent;
    }

    /**
     * 查看非阻塞锁状态
     * @param lockKey 所名称
     * @return 是否存在锁
     */
    private Boolean getNoneBlockingLockStatus(String lockKey){
        return redisTemplate.hasKey(lockKey);
    }

    /**
     * 删除非阻塞锁
     * @param lockKey 锁名称
     */
    public void unlockNoneBlockingLock(String lockKey){
        redisTemplate.delete(lockKey);
    }

    /**
     * 获取阻塞锁
     * @param lockKey 锁名称
     */
    public void getBlockingLock(String lockKey){
        Boolean ifAbsent = redisTemplate.opsForValue().setIfAbsent(lockKey, "",5,TimeUnit.SECONDS);
        while (ifAbsent == null || !ifAbsent){
            try {
                Thread.sleep(100);
            }catch (Exception ignore){}
            ifAbsent = redisTemplate.opsForValue().setIfAbsent(lockKey, "",5,TimeUnit.SECONDS);
        }
    }

    /**
     * 释放阻塞锁
     * @param lockKey 锁名称
     */
    public void unlockBlockingLock(String lockKey){
        redisTemplate.delete(lockKey);
    }
}
