package com.gosingapore.common.util;

import com.cloud.gs.hiring.base.util.ResponseCode;
import com.gosingapore.common.constants.RoleCodeConstants;
import com.gosingapore.common.enums.Dict110ClientIdEnum;
import com.gosingapore.common.enums.dict.Dict111PartiesEnum;
import com.gosingapore.common.exception.CustomizeException;
import com.gosingapore.common.model.dto.UserDetailsImpl;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.util.CollectionUtils;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

/**
 * @author shuyan
 * @date 2020/1/10 13:25
 */
@SuppressWarnings("all")
public class UserUtil {
    private static final String ANONYMOUS = "anonymousUser";

    /**
     * 是否已登录
     *
     * @return
     */
    public static boolean isLogin() {
        SecurityContext context = SecurityContextHolder.getContext();
        if (Objects.isNull(context)) {
            return false;
        }
        Authentication authentication = context.getAuthentication();
        if (Objects.isNull(authentication)) {
            return false;
        }
        Object principal = authentication.getPrincipal();
        return Objects.nonNull(principal);
    }

    /**
     * 获取 userDetails
     *
     * @return 当前用户信息
     */
    public static UserDetailsImpl getUserDetails() {
        SecurityContext context = SecurityContextHolder.getContext();
        if (Objects.isNull(context)) {
            return null;
        }
        Authentication authentication = context.getAuthentication();
        if (Objects.isNull(authentication)) {
            return null;
        }
        Object principal = authentication.getPrincipal();
        if (principal.equals(ANONYMOUS)) {
            return null;
        }
        if (principal instanceof UserDetailsImpl) {
            return (UserDetailsImpl) principal;
        }
        return null;
    }

    /**
     * 获取所属部门id
     *
     * @return 部门id
     */
    public static Integer getDepartmentId() {
        UserDetailsImpl userDetails = getUserDetails();
        if (userDetails == null) {
            return null;
        } else {
            return userDetails.getDepartmentId();
        }
    }

    /**
     * 获取登陆人名字
     * @return
     */
    public static String getName(){
        UserDetailsImpl userDetails = getUserDetails();
        if (userDetails == null){
            return null;
        }else {
            return userDetails.getName();
        }
    }

    /**
     * 获取所属组(主部门)
     * @return 组id
     */
    public static Integer getGroupId(){
        UserDetailsImpl userDetails = getUserDetails();
        if (userDetails == null){
            return null;
        }else {
            return userDetails.getGroupId();
        }
    }

    /**
     * 获取 当前用户Id
     *
     * @return userId
     */
    public static Integer getUserId() {
        UserDetailsImpl userDetails = getUserDetails();
        if (userDetails == null) {
            return null;
        } else {
            return userDetails.getUserId();
        }
    }

    /**
     * 获取客户端类型
     *
     * @return 客户端类型
     */
    public static Dict110ClientIdEnum getClient() {
        UserDetailsImpl userDetails = getUserDetails();
        if (userDetails == null) {
            return null;
        } else {
            return Dict110ClientIdEnum.instanceOf(userDetails.getClient());
        }
    }

    /**
     * 获取当前用户手机号
     *
     * @return 用户手机号
     */
    public static String getMobile() {
        UserDetailsImpl userDetails = getUserDetails();
        if (userDetails == null) {
            return null;
        } else {
            return userDetails.getMobile();
        }
    }

    /**
     * 获取当前用户拥有的角色
     *
     * @return 当前用户拥有的角色
     * @see RoleCodeConstants
     */
    public static List<String> getRoleCodes() {
        UserDetailsImpl userDetails = getUserDetails();
        if (userDetails == null) {
            return null;
        } else {
            return userDetails.getRoleCodes();
        }
    }

    /**
     * 查看是否具有某角色
     *
     * @param roleCode 角色code
     * @return 结果
     * @see RoleCodeConstants
     */
    public static boolean containsRoleCode(String roleCode) {
        UserDetailsImpl userDetails = getUserDetails();
        if (userDetails == null) {
            return false;
        }
        List<String> roleCodes = userDetails.getRoleCodes();
        if (roleCodes == null) {
            return false;
        }
        return roleCodes.contains(roleCode);
    }

    /**
     * 查看是否具有指定任意角色
     *
     * @param roleCodes 角色code
     * @return 结果
     * @see RoleCodeConstants
     */
    public static boolean containsAnyRoleCode(Collection<String> roleCodes) {
        UserDetailsImpl userDetails = getUserDetails();
        if (userDetails == null) {
            return false;
        }
        List<String> roleCode = userDetails.getRoleCodes();
        if (roleCode == null) {
            return false;
        }
        return CollectionUtils.containsAny(roleCode, roleCodes);
    }

    /**
     * 查看是否具有指定任意角色
     *
     * @param roleCodes 角色code
     * @return 结果
     * @see RoleCodeConstants
     */
    public static boolean containsAnyRoleCode(String... roleCodes) {
        UserDetailsImpl userDetails = getUserDetails();
        if (userDetails == null) {
            return false;
        }
        List<String> roleCode = userDetails.getRoleCodes();
        if (roleCode == null) {
            return false;
        }
        return CollectionUtils.containsAny(roleCode, Arrays.asList(roleCodes));
    }

    /**
     * 检查是否包含指定角色中的任意一个
     *
     * @param roleCodes 角色code
     * @see RoleCodeConstants
     */
    public static void checkContainsAnyRoleCode(String... roleCodes) {
        if (!containsAnyRoleCode(roleCodes)) {
            throw new CustomizeException(ResponseCode.PERMISSION_DENIED);
        }
    }

    /**
     * 检查是否包含指定角色中的任意一个
     *
     * @param roleCodes 角色code
     * @see RoleCodeConstants
     */
    public static void checkContainsAnyRoleCode(List<String> roleCodes) {
        if (!containsAnyRoleCode(roleCodes)) {
            throw new CustomizeException(ResponseCode.PERMISSION_DENIED);
        }
    }

    /**
     * 获取当前用户属于方
     *
     * @return
     */
    public static Dict111PartiesEnum getParty() {
        UserDetailsImpl userDetails = getUserDetails();
        if (userDetails == null) {
            return null;
        } else {
            return Dict111PartiesEnum.instanceOf(userDetails.getParty());
        }
    }
}
